import base64


def decode(payload):
    """
    https://en.wikipedia.org/wiki/Base64#URL_applications

    Modified Base64 for URL variants exist, where the + and / characters
    of standard Base64 are respectively replaced by - and _
    """
    return base64.b64decode(payload.replace('-', '+').replace('_', '/'))


def encode(payload):
    """
    https://en.wikipedia.org/wiki/Base64#URL_applications modified Base64
    for URL variants exist, where the + and / characters of standard
    Base64 are respectively replaced by - and _
    """
    if not isinstance(payload, bytes):
        payload = payload.encode('utf-8')

    encoded = base64.b64encode(payload)
    return encoded.replace(b'+', b'-').replace(b'/', b'_').decode('utf-8')
